@@ -2,7 +2,7 @@
  * Copyright (C) 1997-2000 Matt Newman <matt@novadigm.com>
  * Copyright (C) 2000 Ajuba Solutions
  *
- * $Header: /cvsroot/tls/tls/tlsIO.c,v 1.13 2002/02/04 22:46:31 hobbs Exp $
+ * $Header: /cvsroot/tls/tls/tlsIO.c,v 1.16 2007/06/22 21:20:38 hobbs2 Exp $
  *
  * TLS (aka SSL) Channel - can be layered on any bi-directional
  * Tcl_Channel (Note: Requires Trf Core Patch)
@@ -34,7 +34,7 @@ static int	TlsInputProc _ANSI_ARGS_((ClientData instanceData,
 static int	TlsOutputProc _ANSI_ARGS_((ClientData instanceData,
 			CONST char *buf, int toWrite, int *errorCodePtr));
 static int	TlsGetOptionProc _ANSI_ARGS_ ((ClientData instanceData,
-			Tcl_Interp *interp, CONST char *optionName,
+			Tcl_Interp *interp, CONST84 char *optionName,
 			Tcl_DString *dsPtr));
 static void	TlsWatchProc _ANSI_ARGS_((ClientData instanceData, int mask));
 static int	TlsGetHandleProc _ANSI_ARGS_ ((ClientData instanceData,
@@ -122,7 +122,7 @@ Tcl_ChannelType *Tls_ChannelType()
 	 *     against an older variant.
 	 */
 
-	int size = sizeof(Tcl_ChannelType); /* Base size */
+	unsigned int size = sizeof(Tcl_ChannelType); /* Base size */
 
 	/*
 	 * Size of a procedure pointer. We assume that all procedure
@@ -336,6 +336,12 @@ TlsInputProc(ClientData instanceData,	/* Socket state. */
 
     dprintf(stderr,"\nBIO_read(%d)", bufSize);
 
+    if (statePtr->flags & TLS_TCL_CALLBACK) {
+       /* don't process any bytes while verify callback is running */
+       bytesRead = 0;
+       goto input;
+    }
+
     if (!SSL_is_init_finished(statePtr->ssl)) {
 	bytesRead = Tls_WaitForConnect(statePtr, errorCodePtr);
 	if (bytesRead <= 0) {
@@ -414,6 +420,13 @@ TlsOutputProc(ClientData instanceData,	/* Socket state. */
 
     dprintf(stderr,"\nBIO_write(0x%x, %d)", (unsigned int) statePtr, toWrite);
 
+    if (statePtr->flags & TLS_TCL_CALLBACK) {
+       /* don't process any bytes while verify callback is running */
+       written = -1;
+       *errorCodePtr = EAGAIN;
+       goto output;
+    }
+
     if (!SSL_is_init_finished(statePtr->ssl)) {
 	written = Tls_WaitForConnect(statePtr, errorCodePtr);
 	if (written <= 0) {
@@ -507,7 +520,7 @@ TlsOutputProc(ClientData instanceData,	/* Socket state. */
 static int
 TlsGetOptionProc(ClientData instanceData,	/* Socket state. */
 	Tcl_Interp *interp,		/* For errors - can be NULL. */
-	CONST char *optionName,		/* Name of the option to
+	CONST84 char *optionName,	/* Name of the option to
 					 * retrieve the value for, or
 					 * NULL to get all options and
 					 * their values. */
@@ -581,6 +594,12 @@ TlsWatchProc(ClientData instanceData,	/* The socket state. */
 {
     State *statePtr = (State *) instanceData;
 
+    dprintf(stderr, "TlsWatchProc(0x%x)\n", mask);
+
+    /* Pretend to be dead as long as the verify callback is running. 
+     * Otherwise that callback could be invoked recursively. */
+    if (statePtr->flags & TLS_TCL_CALLBACK) { return; }
+
     if (channelTypeVersion == TLS_CHANNEL_VERSION_2) {
 	Tcl_Channel     downChan;
 
